@@ -234,7 +234,7 @@ static
 toff_t
 comp_seek(thandle_t h, toff_t o, int w) {
   io_glue *ig = (io_glue*)h;
-  return (toff_t) ig->seekcb(ig, o, w);
+  return (toff_t) i_io_seek(ig, o, w);
 }
 
 /*
@@ -268,6 +268,21 @@ comp_munmap(thandle_t h, tdata_t p, toff_t off) {
   /* do nothing */
 }
 
+static tsize_t
+comp_read(thandle_t h, tdata_t p, tsize_t size) {
+  return i_io_read((io_glue *)h, p, size);
+}
+
+static tsize_t
+comp_write(thandle_t h, tdata_t p, tsize_t size) {
+  return i_io_write((io_glue *)h, p, size);
+}
+
+static int
+comp_close(thandle_t h) {
+  return i_io_close((io_glue *)h);
+}
+
 static i_img *read_one_tiff(TIFF *tif, int allow_incomplete) {
   i_img *im;
   uint32 width, height;
@@ -547,13 +562,13 @@ i_readtiff_wiol(io_glue *ig, int allow_incomplete, int page) {
   tif = TIFFClientOpen("(Iolayer)", 
 		       "rm", 
 		       (thandle_t) ig,
-		       (TIFFReadWriteProc) ig->readcb,
-		       (TIFFReadWriteProc) ig->writecb,
-		       (TIFFSeekProc) comp_seek,
-		       (TIFFCloseProc) ig->closecb,
-		       ig->sizecb ? (TIFFSizeProc) ig->sizecb : (TIFFSizeProc) sizeproc,
-		       (TIFFMapFileProc) comp_mmap,
-		       (TIFFUnmapFileProc) comp_munmap);
+		       comp_read,
+		       comp_write,
+		       comp_seek,
+		       comp_close,
+		       sizeproc,
+		       comp_mmap,
+		       comp_munmap);
   
   if (!tif) {
     mm_log((1, "i_readtiff_wiol: Unable to open tif file\n"));
@@ -597,7 +612,6 @@ i_readtiff_multi_wiol(io_glue *ig, int *count) {
   TIFFErrorHandler old_warn_handler;
   i_img **results = NULL;
   int result_alloc = 0;
-  int dirnum = 0;
 
   i_clear_error();
   old_handler = TIFFSetErrorHandler(error_handler);
@@ -613,13 +627,13 @@ i_readtiff_multi_wiol(io_glue *ig, int *count) {
   tif = TIFFClientOpen("(Iolayer)", 
 		       "rm", 
 		       (thandle_t) ig,
-		       (TIFFReadWriteProc) ig->readcb,
-		       (TIFFReadWriteProc) ig->writecb,
-		       (TIFFSeekProc) comp_seek,
-		       (TIFFCloseProc) ig->closecb,
-		       ig->sizecb ? (TIFFSizeProc) ig->sizecb : (TIFFSizeProc) sizeproc,
-		       (TIFFMapFileProc) comp_mmap,
-		       (TIFFUnmapFileProc) comp_munmap);
+		       comp_read,
+		       comp_write,
+		       comp_seek,
+		       comp_close,
+		       sizeproc,
+		       comp_mmap,
+		       comp_munmap);
   
   if (!tif) {
     mm_log((1, "i_readtiff_wiol: Unable to open tif file\n"));
@@ -1330,13 +1344,13 @@ i_writetiff_multi_wiol(io_glue *ig, i_img **imgs, int count) {
   tif = TIFFClientOpen("No name", 
 		       "wm",
 		       (thandle_t) ig, 
-		       (TIFFReadWriteProc) ig->readcb,
-		       (TIFFReadWriteProc) ig->writecb,
-		       (TIFFSeekProc)      comp_seek,
-		       (TIFFCloseProc)     ig->closecb, 
-		       ig->sizecb ? (TIFFSizeProc) ig->sizecb : (TIFFSizeProc) sizeproc,
-		       (TIFFMapFileProc)   comp_mmap,
-		       (TIFFUnmapFileProc) comp_munmap);
+		       comp_read,
+		       comp_write,
+		       comp_seek,
+		       comp_close, 
+		       sizeproc,
+		       comp_mmap,
+		       comp_munmap);
   
 
 
@@ -1365,6 +1379,9 @@ i_writetiff_multi_wiol(io_glue *ig, i_img **imgs, int count) {
   TIFFSetErrorHandler(old_handler);
   (void) TIFFClose(tif);
 
+  if (i_io_close(ig))
+    return 0;
+
   return 1;
 }
 
@@ -1398,13 +1415,13 @@ i_writetiff_multi_wiol_faxable(io_glue *ig, i_img **imgs, int count, int fine) {
   tif = TIFFClientOpen("No name", 
 		       "wm",
 		       (thandle_t) ig, 
-		       (TIFFReadWriteProc) ig->readcb,
-		       (TIFFReadWriteProc) ig->writecb,
-		       (TIFFSeekProc)      comp_seek,
-		       (TIFFCloseProc)     ig->closecb, 
-		       ig->sizecb ? (TIFFSizeProc) ig->sizecb : (TIFFSizeProc) sizeproc,
-		       (TIFFMapFileProc)   comp_mmap,
-		       (TIFFUnmapFileProc) comp_munmap);
+		       comp_read,
+		       comp_write,
+		       comp_seek,
+		       comp_close, 
+		       sizeproc,
+		       comp_mmap,
+		       comp_munmap);
   
 
 
@@ -1433,6 +1450,9 @@ i_writetiff_multi_wiol_faxable(io_glue *ig, i_img **imgs, int count, int fine) {
   (void) TIFFClose(tif);
   TIFFSetErrorHandler(old_handler);
 
+  if (i_io_close(ig))
+    return 0;
+
   return 1;
 }
 
@@ -1461,13 +1481,13 @@ i_writetiff_wiol(i_img *img, io_glue *ig) {
   tif = TIFFClientOpen("No name", 
 		       "wm",
 		       (thandle_t) ig, 
-		       (TIFFReadWriteProc) ig->readcb,
-		       (TIFFReadWriteProc) ig->writecb,
-		       (TIFFSeekProc)      comp_seek,
-		       (TIFFCloseProc)     ig->closecb, 
-		       ig->sizecb ? (TIFFSizeProc) ig->sizecb : (TIFFSizeProc) sizeproc,
-		       (TIFFMapFileProc)   comp_mmap,
-		       (TIFFUnmapFileProc) comp_munmap);
+		       comp_read,
+		       comp_write,
+		       comp_seek,
+		       comp_close, 
+		       sizeproc,
+		       comp_mmap,
+		       comp_munmap);
   
 
 
@@ -1487,6 +1507,9 @@ i_writetiff_wiol(i_img *img, io_glue *ig) {
   (void) TIFFClose(tif);
   TIFFSetErrorHandler(old_handler);
 
+  if (i_io_close(ig))
+    return 0;
+
   return 1;
 }
 
@@ -1522,13 +1545,13 @@ i_writetiff_wiol_faxable(i_img *im, io_glue *ig, int fine) {
   tif = TIFFClientOpen("No name", 
 		       "wm",
 		       (thandle_t) ig, 
-		       (TIFFReadWriteProc) ig->readcb,
-		       (TIFFReadWriteProc) ig->writecb,
-		       (TIFFSeekProc)      comp_seek,
-		       (TIFFCloseProc)     ig->closecb, 
-		       ig->sizecb ? (TIFFSizeProc) ig->sizecb : (TIFFSizeProc) sizeproc,
-		       (TIFFMapFileProc)   comp_mmap,
-		       (TIFFUnmapFileProc) comp_munmap);
+		       comp_read,
+		       comp_write,
+		       comp_seek,
+		       comp_close, 
+		       sizeproc,
+		       comp_mmap,
+		       comp_munmap);
   
 
 
@@ -1548,6 +1571,9 @@ i_writetiff_wiol_faxable(i_img *im, io_glue *ig, int fine) {
   (void) TIFFClose(tif);
   TIFFSetErrorHandler(old_handler);
 
+  if (i_io_close(ig))
+    return 0;
+
   return 1;
 }
 
